/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: CMSIS-RTOS2 example program
 * Note(s): possible defines set in "options for target - C/C++ - Define"
 *            __USE_LCD    - enable Output on GLCD
 *----------------------------------------------------------------------------*/

#include "cmsis_os2.h"                  /* ::CMSIS:RTOS2 */
#include "RTE_Components.h"             /* Component selection */
#include CMSIS_device_header
#include "Board_LED.h"                  /* ::Board Support:LED */
#include "Board_GLCD.h"                 /* ::Board Support:Graphic LCD */
#include "GLCD_Config.h"                /* Keil.V2M-MPS2::Board Support:Graphic LCD */

extern GLCD_FONT     GLCD_Font_16x24;

osThreadId_t tid_phaseA;                /* Thread id of thread: phase_a */
osThreadId_t tid_phaseB;                /* Thread id of thread: phase_b */
osThreadId_t tid_phaseC;                /* Thread id of thread: phase_c */
osThreadId_t tid_phaseD;                /* Thread id of thread: phase_d */
osThreadId_t tid_clock;                 /* Thread id of thread: clock */
osThreadId_t tid_lcd;                   /* Thread id of thread: lcd */

osMutexId_t mut_GLCD;                   /* Mutex to controll GLCD access */

#define LED_A      7
#define LED_B      6
#define LED_C      5
#define LED_D      4
#define LED_CLK    0


/*----------------------------------------------------------------------------
 *      Switch LED on
 *---------------------------------------------------------------------------*/
void Switch_On (uint32_t ledNum) {

  LED_On (ledNum);
  osMutexAcquire(mut_GLCD, osWaitForever);
#ifdef __USE_LCD
  GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
  GLCD_SetForegroundColor(GLCD_COLOR_GREEN);
  GLCD_DrawChar((9+ledNum)*16, 6*24, 0x80+1);     /* Circle Full */
#endif
  osMutexRelease(mut_GLCD);
}

/*----------------------------------------------------------------------------
 *      Switch LED off
 *---------------------------------------------------------------------------*/
void Switch_Off (uint32_t ledNum) {

  LED_Off(ledNum);
  osMutexAcquire(mut_GLCD, osWaitForever);
#ifdef __USE_LCD
  GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
  GLCD_SetForegroundColor(GLCD_COLOR_LIGHT_GREY);
  GLCD_DrawChar((9+ledNum)*16, 6*24, 0x80+0);     /* Circle Full               */
#endif
  osMutexRelease(mut_GLCD);
}


/*----------------------------------------------------------------------------
 *      Function 'signal_func' called from multiple threads
 *---------------------------------------------------------------------------*/
void signal_func (osThreadId_t tid)  {
  osThreadFlagsSet(tid_clock, 0x0100);      /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osThreadFlagsSet(tid_clock, 0x0100);      /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osThreadFlagsSet(tid, 0x0001);            /* set signal to thread 'thread' */
  osDelay(500);                             /* delay 500ms                   */
}

/*----------------------------------------------------------------------------
 *      Thread 1 'phaseA': Phase A output
 *---------------------------------------------------------------------------*/
void phaseA (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    Switch_On (LED_A);
    signal_func(tid_phaseB);
    Switch_Off(LED_A);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 2 'phaseB': Phase B output
 *---------------------------------------------------------------------------*/
void phaseB (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    Switch_On (LED_B);
    signal_func(tid_phaseC);
    Switch_Off(LED_B);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 3 'phaseC': Phase C output
 *---------------------------------------------------------------------------*/
void phaseC (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    Switch_On (LED_C);
    signal_func(tid_phaseD);
    Switch_Off(LED_C);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 4 'phaseD': Phase D output
 *---------------------------------------------------------------------------*/
void phaseD (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    Switch_On (LED_D);
    signal_func(tid_phaseA);
    Switch_Off(LED_D);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 5 'clock': Signal Clock
 *---------------------------------------------------------------------------*/
void clock (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0100, osFlagsWaitAny, osWaitForever);
    Switch_On (LED_CLK);
    osDelay(80);
    Switch_Off(LED_CLK);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 6 'lcd': LCD Control thread
 *---------------------------------------------------------------------------*/
void lcd (void *argument) {

  for (;;) {
    osMutexAcquire(mut_GLCD, osWaitForever);
#ifdef __USE_LCD
  GLCD_SetBackgroundColor(GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor(GLCD_COLOR_WHITE);
  GLCD_DrawString(0*16, 0*24, "   V2M-MPS2 Demo    ");
  GLCD_DrawString(0*16, 1*24, "      Blinky        ");
  GLCD_DrawString(0*16, 2*24, "   www.keil.com     ");
#endif
    osMutexRelease(mut_GLCD);
    osDelay(4000);

    osMutexAcquire(mut_GLCD, osWaitForever);
#ifdef __USE_LCD
  GLCD_SetBackgroundColor(GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor(GLCD_COLOR_RED);
  GLCD_DrawString(0*16, 0*24, "   V2M-MPS2 Demo    ");
  GLCD_DrawString(0*16, 1*24, "      Blinky        ");
  GLCD_DrawString(0*16, 2*24, "   www.keil.com     ");
#endif
    osMutexRelease(mut_GLCD);
    osDelay(4000);
  }
}


/*----------------------------------------------------------------------------
 *      Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
void app_main (void *argument) {

  mut_GLCD = osMutexNew(NULL);

  tid_phaseA = osThreadNew(phaseA, NULL, NULL);
  tid_phaseB = osThreadNew(phaseB, NULL, NULL);
  tid_phaseC = osThreadNew(phaseC, NULL, NULL);
  tid_phaseD = osThreadNew(phaseD, NULL, NULL);
  tid_clock  = osThreadNew(clock,  NULL, NULL);
  tid_lcd    = osThreadNew(lcd, NULL, NULL);

  osThreadFlagsSet(tid_phaseA, 0x0001);     /* set signal to phaseA thread */

  osDelay(osWaitForever);
}

int main (void) {

  SystemCoreClockUpdate();
  LED_Initialize();

#ifdef __USE_LCD
  GLCD_Initialize();

  GLCD_SetFont(&GLCD_Font_16x24);
  GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
  GLCD_ClearScreen();

  GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
  GLCD_SetForegroundColor(GLCD_COLOR_BLACK);
  switch ((SCB->CPUID >> 4) & 0xFFF) {
    case 0xC20:
      GLCD_DrawString(0*16, 4*24, "   ARM Cortex-M0    ");
      break;
    case 0xC60:
      GLCD_DrawString(0*16, 4*24, "   ARM Cortex-M0+   ");
      break;
    case 0xC21:
      GLCD_DrawString(0*16, 4*24, "   ARM Cortex-M1    ");
      break;
    case 0xC23:
      GLCD_DrawString(0*16, 4*24, "   ARM Cortex-M3    ");
      break;
    case 0xC24:
      GLCD_DrawString(0*16, 4*24, "   ARM Cortex-M4    ");
      break;
    case 0xC27:
      GLCD_DrawString(0*16, 4*24, "   ARM Cortex-M7    ");
      break;
    default:
      GLCD_DrawString(0*16, 4*24, "  unknown Cortex-M  ");
      break;
  }
#endif

  osKernelInitialize();                     /* Initialize CMSIS-RTOS */
  osThreadNew(app_main, NULL, NULL);        /* Create application main thread */
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                        /* Start thread execution */
  }

  while(1);
}
